/*
 *  Yet another ADO wrapper class
 *  (c) 2008 Carlos Mora (carlosantoniomora@yahoo.es - harbouradvisor.blogspot.com )
 *  based and inspired in the work of Jos Luis Capel and Gabriel Maim (Biel)
 *
 */


#include "MyInclude.ch"
#include "hbclass.ch"
#include "ado.ch"

//--------------------------------------------------------------------------
CLASS TAdoConnection
//--------------------------------------------------------------------------

   DATA cConnString        AS CHARACTER
   DATA cUser              AS CHARACTER
   DATA cPass              AS CHARACTER
   DATA nOptions
   DATA lFromUser          AS LOGICAL

   DATA oConn              AS OBJECT
   DATA oRS                AS OBJECT

   METHOD New( cString, cUser, cPassword, nOptions, lFromUser ) CONSTRUCTOR
   METHOD Open()
   METHOD Close()
   METHOD Execute( cCommand, nCount, nOptions )

ENDCLASS


//--------------------------------------------------------------------------
METHOD New( cString, cUser, cPassword, nOptions, lFromUser ) CLASS TAdoConnection
//--------------------------------------------------------------------------
   LOCAL oError, lResult

   lResult := .T.

   TRY
      ::oConn := CreateObject( "ADODB.Connection" )
   CATCH oError
      lResult := .F.
      Alert( oError:Operation, "Ado Connection" )
   END

   IF lResult
      DEFAULT lFromUser TO Empty( cString )
      ::cConnString := cString
      ::cUser       := cUser
      ::cPass       := cPassword
      ::nOptions    := nOptions
      ::lFromUser   := lFromUser

      ::oConn:CommandTimeOut    := 5
      ::oConn:ConnectionTimeOut := 10
      ::oConn:CursorLocation    := adUseClient

   ENDIF

RETURN Self


//--------------------------------------------------------------------------
METHOD Open() CLASS TAdoConnection
//--------------------------------------------------------------------------
   LOCAL oError, lResult:= .T.
   LOCAL oDataLink

   If ::oConn != NIL
      If ::oConn:State != adStateClosed
         ::oConn:Close()
      EndIf
   EndIf

   IF ::lFromUser    // Crearlo via Datalinks

      oDataLink := TOleAuto():New("Datalinks")
      ::oConn   := oDataLink:PromptNew()
      lResult   := ( ::oConn != NIL )

   ELSE

      TRY
        ::oConn:Open( ::cConnString, ::cUser, ::cPass, ::nOptions )
      CATCH oError
         lResult := .F.
         Alert( oError:Operation+CRLF+oError:Description ) // , "Ado Connection" )
      END

   ENDIF

RETURN lResult

//--------------------------------------------------------------------------
METHOD Close() CLASS TAdoConnection
//--------------------------------------------------------------------------
   LOCAL oError, lResult := .T.

   If ::oRS != NIL
      If ::oRS:State == adStateOpen
         ::oRS:Close()
         ::oRS:= NIL
      EndIf
   EndIf

   TRY
      ::oConn:Close()
      ::oConn:= NIL
   CATCH oError
      lResult := .F.
      Alert( oError:Operation+HB_OSNewLine()+oError:Description, "Ado Connection" )
      QUIT
   END

RETURN lResult



METHOD Execute( cCommand, nRecAff, nOptions ) CLASS TAdoConnection
   LOCAL oError, lResult:= .T.
   LOCAL oRS
   DEFAULT nRecAff   TO 0

   lResult := .T.

   /* A closed Recordset object will be returned if it is not a row-returning query.
      The returned Recordset is always a read-only, forward-only Recordset!
      To create a Recordset with more functionality, first create a Recordset object.
      Set the desired properties, and then use the Recordset object's Open method to execute the query.
   */

   If ::oRS != NIL
      If ::oRS:State == adStateOpen
         ::oRS:Close()
         ::oRS:= NIL
      EndIf
   EndIf

   TRY
      ::oRS:= ::oConn:Execute( cCommand, @nRecAff, nOptions)
   CATCH oError
      lResult := .F.
      Alert( oError:Operation+CRLF+oError:Description, "Ado Connection" )
      quit
   END

RETURN lResult


/*
Connection Object

The ADO Connection Object is used to create an open connection to a data source. Through this connection, you can access and manipulate a database.

If you want to access a database multiple times, you should establish a connection using the Connection object. You can also make a connection to a database by passing a connection string via a Command or Recordset object. However, this type of connection is only good for one specific, single query.
ProgID

set objConnection=Server.CreateObject("ADODB.connection")

Properties
Property          Description
Attributes        Sets or returns the attributes of a Connection object
CommandTimeout    Sets or returns the number of seconds to wait while attempting to execute a command
ConnectionString  Sets or returns the details used to create a connection to a data source
ConnectionTimeout Sets or returns the number of seconds to wait for a connection to open
CursorLocation    Sets or returns the location of the cursor service
DefaultDatabase   Sets or returns the default database name
IsolationLevel    Sets or returns the isolation level
Mode              Sets or returns the provider access permission
Provider          Sets or returns the provider name
State             Returns a value describing if the connection is open or closed
Version           Returns the ADO version number

Methods
Method         Description
BeginTrans     Begins a new transaction
Cancel         Cancels an execution
Close          Closes a connection
CommitTrans    Saves any changes and ends the current transaction
Execute        Executes a query, statement, procedure or provider specific text
Open           Opens a connection
OpenSchema     Returns schema information from the provider about the data source
RollbackTrans  Cancels any changes in the current transaction and ends the transaction
Events

Note:  You cannot handle events using VBScript or JScript (only Visual Basic, Visual C++, and Visual J++ languages can handle events).
Event    Description
BeginTransComplete   Triggered after the BeginTrans operation
CommitTransComplete  Triggered after the CommitTrans operation
ConnectComplete   Triggered after a connection starts
Disconnect  Triggered after a connection ends
ExecuteComplete   Triggered after a command has finished executing
InfoMessage    Triggered if a warning occurs during a ConnectionEvent operation
RollbackTransComplete   Triggered after the RollbackTrans operation
WillConnect    Triggered before a connection starts
WillExecute    Triggered before a command is executed
Collections
Collection  Description
Errors   Contains all the Error objects of the Connection object
Properties  Contains all the Property objects of the Connection object
*/